
import time
from subprocess import check_output, CalledProcessError


class WA_SQLITE(object):
    """
    Class with references to create the query strings needed to be executed through ADB 
    in order to perform actions in androids' whatsapp.
    """

    K_CONSTANT = "-1150867590"

    DATABASE_PATH = "/data/data/com.whatsapp/databases/msgstore.db"

    MESSAGES_TABLE_NAME = "messages"

    CHAT_LIST_TABLE_NAME = "chat_list"

    INSERT_INTO_MESSAGES_QUERY = "INSERT INTO {messages_table_name} "\
    "(key_remote_jid, key_from_me, key_id, status, needs_push, data, timestamp, MEDIA_URL, media_mime_type, media_wa_type, MEDIA_SIZE, media_name , latitude, longitude, thumb_image, remote_resource, received_timestamp, send_timestamp, receipt_server_timestamp, receipt_device_timestamp, raw_data, media_hash, recipient_count, media_duration, origin) "\
    "VALUES ('{remote_jid}', 1, '{key_id_to}-{k_constant}', 0, 0, '{message}', {key_id_from}, '', '', 0, 0, '', 0.0, 0.0, '', '', {key_id_from}, -1, -1, -1, 0 , '', 0, 0, 0);"

    INSERT_INTO_CHATLIST_QUERY = "INSERT INTO {chatlist_table_name} (key_remote_jid) "\
    "SELECT '{remote_jid}' WHERE not exists (SELECT 1 from chat_list where key_remote_jid='{remote_jid}');"

    UPDATE_CHATLIST_QUERY = "UPDATE {chatlist_table_name} SET message_table_id = "\
    "(SELECT max(messages._id) FROM messages) WHERE chat_list.key_remote_jid='{remote_jid}';"

    GET_MESSAGES = "SELECT _id, key_remote_jid, key_from_me, key_id, status, data, timestamp FROM {messages_table_name}"
    
    FILTER_FROM_JID = " key_remote_jid = '{key_remote_jid}' "

    FILTER_FROM_DATE = " timestamp>={filter_from_date_epoch} "

    FILTER_TO_DATE = " timestamp<={filter_to_date_epoch} "

    def __init__(self, remote_jid):
        """
        """
        self.remote_jid = remote_jid
        pass

    @classmethod
    def get_key_id_to_and_from(cls):
        """
        """
        key_id_from=int(round(time.time() * 1000))
        key_id_to=int(key_id_from / 1000)
        return  (key_id_from, key_id_to)

    @classmethod
    def get_insert_into_messages_query(cls, remote_jid, message):
        """
        """
        key_id_from, key_id_to = cls.get_key_id_to_and_from()
        return cls.INSERT_INTO_MESSAGES_QUERY.format(
            remote_jid=remote_jid,
            messages_table_name=cls.MESSAGES_TABLE_NAME,
            key_id_from=key_id_from,
            key_id_to=key_id_to,
            k_constant=cls.K_CONSTANT,
            message=message,

        )
    
    @classmethod
    def get_insert_into_chat_list_query(cls, remote_jid):
        """
        """
        return cls.INSERT_INTO_CHATLIST_QUERY.format(
            remote_jid=remote_jid,
            chatlist_table_name=cls.CHAT_LIST_TABLE_NAME
        )

    @classmethod
    def get_update_chatlist_query(cls, remote_jid):
        """
        """
        return cls.UPDATE_CHATLIST_QUERY.format(
            remote_jid=remote_jid,
            chatlist_table_name=cls.CHAT_LIST_TABLE_NAME
        )
    
    @classmethod
    def get_messages(cls, key_remote_jid=None, from_date=None, to_date=None):
        """
        """
        query_string = cls.GET_MESSAGES.format(messages_table_name=cls.MESSAGES_TABLE_NAME)
        if key_remote_jid or from_date or to_date:
            query_string += " WHERE"
            filter_list = []
            if key_remote_jid:
                filter_list.append(cls.FILTER_FROM_JID.format(key_remote_jid=key_remote_jid))
            if from_date:
                filter_list.append(cls.FILTER_FROM_DATE.format(filter_from_date_epoch=from_date))
            if to_date:
                filter_list.append(cls.FILTER_TO_DATE.format(filter_to_date_epoch=to_date))
            query_string += "AND".join(filter_list)    
        query_string += ";"
        return query_string

    # re split...
    

    pass


from enum import Enum


class Who(Enum):
    FROM_ME = "key_from_me == 1"
    OTHERS = "key_from_me != 1"
    ALL = ""
    pass


class Type(Enum):
    GPS = 1
    IMAGE = 2
    TEXT = 3


class COLUMN(Enum):
    _id = id = 0
    key_remote_jid = 1
    key_from_me = 2
    key_id = 3
    status = 4
    needs_push = 5
    data = 6
    message = 6
    timestamp = 7
    media_url = 8
    media_mime_type = 9
    media_wa_type = 10
    media_size = 11
    media_name = 12
    media_caption = 13
    media_hash = 14
    media_duration = 15
    origin = 16
    latitude = 17
    longitude = 18
    thumb_image = 19
    remote_resource = 20
    received_timestamp = 21
    send_timestamp = 22
    receipt_server_timestamp = 23
    receipt_device_timestamp = 24
    read_device_timestamp = 25
    played_device_timestamp = 26
    raw_data = 27
    recipient_count = 28
    participant_hash = 29
    starred = 30
    quoted_row_id = 31
    mentioned_jids = 32
    multicast_id = 33
    edit_version = 34
    media_enc_hash = 35
    payment_transaction_id = 36





import re

from subprocess import check_output, CalledProcessError
from py_wa_adb.wa_sqlite import WA_SQLITE
import time

def datetime_to_epoch(datetime_obj):
    return int(float(datetime_obj.strftime("%s"))*1000)



class WA_ADB(object):
    """
    """

    WHATSAPP_ANDROID_PACKAGE_STRING = "com.whatsapp"

    MAIN_ACTIVITY_STRING_REFERENCE = ".Main"

    MESSAGES_ROW_REGEX = \
    rb"(?P<_id>\d+)\|(?P<key_remote_jid>.*?)\|(?P<key_from_me>\d+)\|(?P<key_id>.*?)\|(?P<status>\d+)\|(?P<data>[\s\S]+?)\|(?P<timestamp>\d+)\n"
    
    def __init__(self):
        pass

    def wrap_adb_shell(self, command_string, wrap=True, device=None):
        """
        """
        device = "" if device is None else "-s {device} "
        if wrap:
            return f"adb {device}shell \"{command_string}\""
        return f"adb {device}shell {command_string}"


    def get_kill_whatsapp_processes_string(self):
        """
        """
        return self.wrap_adb_shell(f"pkill {self.WHATSAPP_ANDROID_PACKAGE_STRING}")

    def get_am_start_application(self):
        return self.wrap_adb_shell(f"am start -n {self.WHATSAPP_ANDROID_PACKAGE_STRING}/{self.MAIN_ACTIVITY_STRING_REFERENCE}")

    def get_change_wa_db_perms(self):
        return self.wrap_adb_shell(f"chmod 777 {WA_SQLITE.DATABASE_PATH}")
    
    def get_insert_into_messages_query(self, wa_number_id, message):
        return self.wrap_adb_shell(f"sqlite3 {WA_SQLITE.DATABASE_PATH} \\\"{WA_SQLITE.get_insert_into_messages_query(wa_number_id, message)}\\\"")
        pass

    def get_insert_into_chat_list_query(self, wa_number_id):
        return self.wrap_adb_shell(f"sqlite3 {WA_SQLITE.DATABASE_PATH} \\\"{WA_SQLITE.get_insert_into_chat_list_query(wa_number_id)}\\\"")
        pass

    def get_update_chatlist_query(self, wa_number_id):
        return self.wrap_adb_shell(f"sqlite3 {WA_SQLITE.DATABASE_PATH} \\\"{WA_SQLITE.get_update_chatlist_query(wa_number_id)}\\\"")
    
    def get_read_messages_query(self, **kwargs):
        """
        """
        return self.wrap_adb_shell(f"sqlite3 {WA_SQLITE.DATABASE_PATH} \\\"{WA_SQLITE.get_messages(**kwargs)}\\\"")
        pass

    def send_text_message(self, wa_number_id, message, device=None):
        """

        """
        check_output(self.get_kill_whatsapp_processes_string(), shell=True)
        check_output(self.get_change_wa_db_perms(), shell=True)
        
        check_output(self.get_insert_into_messages_query(wa_number_id, message), shell=True)
        for _ in range(5):
            try:
                check_output(self.get_insert_into_chat_list_query(wa_number_id), shell=True)
                break
            except Exception as e:
                print(e)
                time.sleep(3)
                continue
        for _ in range(5):
            try:
                check_output(self.get_update_chatlist_query(wa_number_id), shell=True)
            except Exception as e:
                print(e)
                time.sleep(3)
                continue
                
        check_output(self.get_am_start_application(), shell=True)

        pass

    def get_messages(self, parse_rows=True, **kwargs):
        """
        """
        
        sqlite_query_output = check_output(self.get_read_messages_query(**kwargs), shell=True)
        if parse_rows:
            return re.findall(self.MESSAGES_ROW_REGEX, sqlite_query_output)
        return sqlite_query_output
    

    pass




import argparse
import sys
from py_wa_adb.wa_adb import WA_ADB, datetime_to_epoch

class WA_ADB_CLI_PARSER(object):
    """
    """
    def __init__(self):
        """
        """
        self.parser = argparse.ArgumentParser(description='Process some integers.')
        self.subparsers = self.parser.add_subparsers()
        self.send_message_parser = self.subparsers.add_parser("send_message")
        self.send_message_parser.set_defaults(which="send_text_message")
        self.send_message_parser.add_argument("wa_number_id")
        self.send_message_parser.add_argument("message")
        self.send_message_parser.add_argument("--device", required=False)

        
        self.read_messages_parser = self.subparsers.add_parser("get_messages")
        self.read_messages_parser.set_defaults(which="get_messages")
        
        self.read_messages_parser.add_argument("--key_remote_jid", required=False)
        self.read_messages_parser.add_argument("--from_date", required=False)
        self.read_messages_parser.add_argument("--to_date", required=False)
        
        # get_messages
        # self.read_messages_parser.set_defaults(which="read_messages_parser")

        pass

    pass

def main():
    adb_cli_parser = WA_ADB_CLI_PARSER()
    x = adb_cli_parser.parser.parse_args()

    if not hasattr(x, "which"):
        print("MISSING ARGUMENT")
        sys.exit(-1)
        pass
    wa_adb = WA_ADB()
    # Call expected method and remove which from the passed args.
    print(getattr(wa_adb, x.which)(
        **{y[0]: y[1] for y in x._get_kwargs() if y[0] != "which"}
    ))
    pass



if __name__ == "__main__":
    main()